@@ -931,14 +931,19 @@ class Engine:
 
     def update(
             self,
-            interval: float
+            interval: float,
+            global_time_precision: Optional[int] = None
     ) -> None:
         """
         Run each process for the given interval and force them to complete
-        at the end of the interval.
+        at the end of the interval. See ``run_for`` for the keyword args.
         """
         clock_start = clock.time()
-        self.run_for(interval=interval, force_complete=True)
+        self.run_for(
+            interval=interval,
+            force_complete=True,
+            global_time_precision=global_time_precision
+        )
         self._check_complete()
         runtime = clock.time() - clock_start
         if self.display_info:
@@ -962,6 +967,7 @@ class Engine:
             self,
             interval: float,
             force_complete: bool = False,
+            global_time_precision: Optional[int] = None
     ) -> None:
         """Run each process within the given interval and update their states.
 
@@ -969,6 +975,9 @@ class Engine:
             interval: the amount of time to simulate the composite.
             force_complete: a bool indicating whether to force processes
                 to complete at the end of the interval.
+            global_time_precision: an optional int that sets the decimal
+                precision of global_time. This is useful for remove floating-
+                point rounding errors for the time keys of saved states.
         """
         end_time = self.global_time + interval
         emit_time = self.global_time + self.emit_step
@@ -1013,6 +1022,9 @@ class Engine:
                         future = min(process_time + process_timestep, end_time)
                     else:
                         future = process_time + process_timestep
+                    if global_time_precision is not None:
+                        # set future time based on global_time_precision
+                        future = round(future, global_time_precision)
 
                     if future <= end_time:
                         # calculate the update for this process
